@@ -21,11 +21,11 @@ class SubprocessError(EnvironmentError):
 
 from Updater import Updater
 
-process_group_pid_list = []
+process_group_pid_set = set()
 process_pid_file_list = []
 process_command_list = []
 def sigterm_handler(signal, frame):
-  for pgpid in process_group_pid_list:
+  for pgpid in process_group_pid_set:
     try:
       os.killpg(pgpid, signal.SIGTERM)
     except:
@@ -96,6 +96,12 @@ repository = %(repository_path)s
   retry_software = False
   try:
     while True:
+      # kill processes from previous loop if any
+      for pgpid in process_group_pid_set:
+        try:
+          os.killpg(pgpid, signal.SIGTERM)
+        except:
+          pass
       # Make sure we have local repository
       if not os.path.exists(repository_path):
         parameter_list = [config['git_binary'], 'clone',
@@ -143,11 +149,11 @@ repository = %(repository_path)s
 
         # Now prepare the installation of SlapOS
         slapos_controler = SlapOSControler(config,
-          process_group_pid_list=process_group_pid_list)
+          process_group_pid_set=process_group_pid_set)
         # this should be always true later, but it is too slow for now
         status_dict = slapos_controler.runSoftwareRelease(config,
           environment=config['environment'],
-          process_group_pid_list=process_group_pid_list,
+          process_group_pid_set=process_group_pid_set,
           )
         if status_dict['status_code'] != 0:
           safeRpcCall(master.reportTaskFailure,
@@ -157,7 +163,7 @@ repository = %(repository_path)s
 
         # create instances, it should take some seconds only
         slapos_controler.runComputerPartition(config,
-                process_group_pid_list=process_group_pid_list)
+                process_group_pid_set=process_group_pid_set)
 
         # update repositories downloaded by buildout. Later we should get
         # from master a list of repositories
@@ -196,15 +202,16 @@ repository = %(repository_path)s
                          '--node_quantity', config['node_quantity'],
                          '--master_url', config['test_suite_master_url'],
                          ], )
-        process_group_pid_list.append(run_test_suite.pid)
+        process_group_pid_set.add(run_test_suite.pid)
         run_test_suite.wait()
+        process_group_pid_set.remove(run_test_suite.pid)
 
   finally:
     # Nice way to kill *everything* generated by run process -- process
     # groups working only in POSIX compilant systems
     # Exceptions are swallowed during cleanup phase
-    print "going to kill %r" % (process_group_pid_list,)
-    for pgpid in process_group_pid_list:
+    print "going to kill %r" % (process_group_pid_set,)
+    for pgpid in process_group_pid_set:
       try:
         os.killpg(pgpid, signal.SIGTERM)
       except:
